// ====================================================================
// Copyright (C) 2011 - Tsubasa Saito
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// ====================================================================

// *****************************************************************
// Inner product of QD numbers
//
// Calling Sequence :
// qdip(x,y)
//
// Arguments :
// x,y : QD vectors
//
// Authors :
// Copyright (C) 2011 - Tsubasa Saito
// last updating : 2011/11/09
// *****************************************************************
function z = qdip(x,y)
  [m1,n1] = size(x.d);
  [m2,n2] = size(y.d);
  if n1 ~= 1 | n2 ~=1 | m1 ~= m2 then
    disp('Error in  qdip!');
    z = tlist(['qd','d'],0,0,0,0);
  else
    z = tlist(['qd','d'],0,0,0,0);
    //[c1,c2,c3,c4] = x(:,1).*y(:,1);
    [p1,e1] = two_prod(x(2)(:,1),y(2)(:,1));
    [p2,e2] = two_prod(x(2)(:,1),y(3)(:,1));
    [p3,e3] = two_prod(x(3)(:,1),y(2)(:,1));
    [p4,e4] = two_prod(x(2)(:,1),y(4)(:,1));
    [p5,e5] = two_prod(x(3)(:,1),y(3)(:,1));
    [p6,e6] = two_prod(x(4)(:,1),y(2)(:,1));
    //O(eps) terms
    [p2,p3,e1] = three_sum(p2,p3,e1);
    //O(eps^2) terms
    [p4,p5,p6] = three_sum(p4,p5,p6);
    [p3,e2,e3] = three_sum(p3,e2,e3);
    [p3,p4] = two_sum(p3,p4);
    [p5,e2] = two_sum(e2,p5);
    [p4,p5] = two_sum(p4,p5);
    p5 = p5+e2+e3+p6;
    //O(eps^3) terms
    p4 = p4+x(2)(:,1).*y(5)(:,1)+x(3)(:,1).*y(4)(:,1)+x(4)(:,1).*y(3)(:,1)+x(5)(:,1).*y(2)(:,1)+e4+e5+e6;
    [c1,c2,c3,c4] = renormalize(p1,p2,p3,p4,p5);
    for i=1:m1
        //z = z + [c1,c2,c3,c4]
        [s1,t1] = two_sum(z(2),c1(i,1));
        [s2,t2] = two_sum(z(3),c2(i,1));
        [s3,t3] = two_sum(z(4),c3(i,1));
        [s4,t4] = two_sum(z(5),c4(i,1));
        [s2,t1] = two_sum(s2,t1);
        [s3,t1,t2] = three_sum(s3,t2,t1);
        [s4,t1] = three_sum2(s4,t3,t1);
        s5 =t1+t2+t4;
        [t1,t2,t3,t4] = renormalize(s1,s2,s3,s4,s5);
        z = tlist(['qd','d'],t1,t2,t3,t4);
    end
  end
endfunction

